//MoveNetの変数
let videoPlayer;
let detector;
let overlayCanvas;
let overlayCtx;

//旗揚げゲームの変数
const patterns = [
    ['{色}あげて', ''],
    ['{色}さげて', ''],

    ['{色}あげないで', '{色}あげて'],
    ['{色}あげないで', '{色}さげて'],
    ['{色}あげないで', '{色}あげない'],
    ['{色}あげないで', '{色}さげない'],

    ['{色}さげないで', '{色}あげて'],
    ['{色}さげないで', '{色}さげて'],
    ['{色}さげないで', '{色}あげない'],
    ['{色}さげないで', '{色}さげない'],
];

var redState = false; // 赤旗の初期状態
var whiteState = false; // 白旗の初期状態

const state = [
    ['あげて', true],
    ['さげて', false],
    ['あげない', false],
    ['さげない', true],
];

// ゲームの状態管理
let count = 0; // ゲームの回数
let miss = 0; // ミスの数
let right, left; // 手の判定結果
let poseData; //姿勢データ


// MoveNetモデルを読み込む関数
async function loadMoveNet() {
    try {
        await tf.setBackend('webgl');
        detector = await poseDetection.createDetector(
            poseDetection.SupportedModels.MoveNet,
            {
                modelType: poseDetection.movenet.modelType.SINGLEPOSE_LIGHTNING,
            }
        );
    } catch (error) {
        console.error('MoveNet モデルの読み込みに失敗しました:', error);
    }
}


// ビデオからポーズを推定する関数
async function estimatePoses(video) {
    try {
        const poses = await detector.estimatePoses(video);
        return poses;
    } catch (error) {
        console.error('姿勢推定に失敗しました:', error);
        return [];
    }
}


// カメラの読み込みを行う関数
async function loadCamera() {
    const constraints = {
        video: {
            width: 1980,
            height: 1080,
            aspectRatio: 1.77
        }
    };
    try {
        const stream = await navigator.mediaDevices.getUserMedia(constraints);
        videoPlayer.srcObject = stream;
    } catch (error) {
        console.error('カメラのアクセスに失敗:', error);
    }
}


// 選択した動画を再生し、ポーズ検出を開始する関数
async function playSelectedVideo() {
    await videoPlayer.play();
    detect();
}


// キーポイントを描画する関数（オーバーレイ用）
function drawKeypoints(poses, context) {
    poses.forEach((pose) => {
        pose.keypoints.forEach((keypoint) => {
            // キーポイントnoseの座標を取得し、追従する画像を描画
            if (keypoint.name == 'right_wrist') {
                drawRightFlagImage(keypoint.x, keypoint.y, context);
            }
            if (keypoint.name == 'left_wrist') {
                drawLeftFlagImage(keypoint.x, keypoint.y, context);
            }
        });
    });
}

// 右手の旗を描画する関数
function drawRightFlagImage(x, y, context) {
    if (rightFlag.complete) {
        const imageWidth = 300;
        const imageHeight = 300;
        context.drawImage(rightFlag, x - imageWidth / 2, y - imageHeight / 2, imageWidth, imageHeight);
    }
}


// 左手の旗を描画する関数
function drawLeftFlagImage(x, y, context) {
    if (leftFlag.complete) {
        const imageWidth = 300;
        const imageHeight = 300;
        context.drawImage(leftFlag, x - imageWidth / 2, y - imageHeight / 2, imageWidth, imageHeight);
    }
}


// 画像をロードする関数
function loadfaceImage() {
    rightFlag = new Image();
    rightFlag.src = 'img/flag_red.png';
    leftFlag = new Image();
    leftFlag.src = 'img/flag_white.png';
}


// ポーズを検出し続ける関数
async function detect() {
    try {
        if (!videoPlayer.paused && !videoPlayer.ended) {
            const poses = await estimatePoses(videoPlayer);
            poseData = poses;
            if (poses.length > 0) {
                // オーバーレイ用のキャンバスをクリアしてキーポイントを描画
                overlayCtx.clearRect(0, 0, overlayCanvas.width, overlayCanvas.height);
                drawKeypoints(poses, overlayCtx);
            }
        }
    } catch (error) {
        console.error('姿勢推定に失敗しました:', error);
    }
    requestAnimationFrame(detect);
}


// ページの読み込みが完了したらMoveNetモデルを読み込み、要素を取得する
window.onload = async function () {
    await loadMoveNet();
    videoPlayer = document.getElementById('videoPlayer');
    overlayCanvas = document.getElementById('overlayCanvas');
    overlayCtx = overlayCanvas.getContext('2d');
    loadfaceImage(); // 追従する画像をロードする

    // カメラ映像の読み込み
    await loadCamera();

    videoPlayer.addEventListener('loadedmetadata', function () {
        playSelectedVideo();
        setTimeout(startGame, 6000);
    });
};


function randomPattern() {
    const colors = ['赤', '白']; // 色のリスト
    const chosenColors = colors.map(color => color); // 色のコピーを作成

    const chosenPattern = patterns[Math.floor(Math.random() * patterns.length)]; // ランダムにパターンを選択

    // パターン内の色をランダムに置換して表示
    const result = chosenPattern.map(pattern => pattern.replace('{色}', chosenColors[Math.floor(Math.random() * chosenColors.length)]));
    return result;
}


function isFlagRised() {
    // ランダムな指示パターンを取得する
    const [instruction_1, instruction_2] = randomPattern();

    // 指示から状態を更新する関数
    const updateState = (instruction) => {
        // 指示が存在しない場合は何もしない
        if (!instruction) return;

        // 指示から色と名前を抽出
        const result = instruction.match(/(赤|白)(.+)/);
        const isRed = result[1].includes('赤'); // 赤か白かを判断
        const stateObj = state.find(([name]) => result[2].includes(name)); // 名前に一致する状態を探す
        const stateValue = stateObj ? stateObj[1] : null; // 状態の値を取得 (存在しない場合はnull)

        // 状態を更新
        if (isRed) {
            redState = stateValue; // 赤の場合は赤の状態を更新
        } else {
            whiteState = stateValue; // 白の場合は白の状態を更新
        }
    }

    // 2つの指示から状態を更新
    updateState(instruction_1);
    updateState(instruction_2);

    // 指示と最終的な状態を返す
    return [instruction_1, instruction_2, redState, whiteState];
}


// 手の判定を行う
function judgeHands(flagState) {
    // 右手の判定
    const rightHandY = poseData[0].keypoints[10].y <= 540 ? true : false;
    const rightHandCondition = flagState[2];
    right = (rightHandY && rightHandCondition) || (!rightHandY && !rightHandCondition)
        ? 'success'
        : 'fail';
    console.log(`右手: ${right}`);

    // 左手の判定
    const leftHandY = poseData[0].keypoints[9].y <= 540 ? true : false;
    const leftHandCondition = flagState[3];
    left = (leftHandY && leftHandCondition) || (!leftHandY && !leftHandCondition)
        ? 'success'
        : 'fail';
    console.log(`左手: ${left}`);

    // ミスの集計
    if (right === 'fail' || left === 'fail') {
        console.log('減点！');
        miss++;
    }

    const missDisplay = document.getElementById('miss');
    missDisplay.textContent = `ミスの数: ${miss}`;

    // ゲームの継続判定
    const order = document.getElementById('order');
    if (count < 10 && miss < 3) {
        startGame(); // 次の回へ
    } else if (miss >= 3) {
        order.textContent = 'ゲームオーバー！';
    } else {
        order.textContent = 'ゲームクリア！';
    }
}

// ゲームの開始
function startGame() {
    console.log(`${count + 1}回目`);
    count++;

    const flagState = isFlagRised();
    const order = document.getElementById('order');
    order.textContent = flagState[0] + flagState[1];

    // 3秒待機後に手の判定を行う
    setTimeout(judgeHands, 3000, flagState);
}